import strawberry
from tarandm_analytics_utils.utils.base_model import BaseModel as BaseModel

class AttributeTransformation(BaseModel):
    attribute: str
    transformation: str
    transformed_attribute_name: str | None

class AttributeTransformationType:
    attribute: strawberry.auto
    transformation: strawberry.auto
    transformed_attribute_name: strawberry.auto



import strawberry
from enum import Enum
from tarandm_analytics_utils.utils.base_model import BaseModel as BaseModel

class SampleType(str, Enum):
    TRAIN = 'TRAIN'
    VALID = 'VALID'
    TEST = 'TEST'
    OOT = 'OOT'
    CV = 'CV'

class PerformanceMetrics(str, Enum):
    AUC = 'AUC'
    GINI = 'GINI'
    LIFT = 'LIFT'
    KS = 'KS'
    TAU = 'TAR'
    RHO = 'RHO'
    ACCURACY = 'ACCURACY'

class SamplePerformance(BaseModel):
    sample: SampleType
    target: str
    performance: dict[PerformanceMetrics, float]

class SamplePerformancePerformanceType:
    metric: PerformanceMetrics
    value: float

class SamplePerformanceType:
    sample: strawberry.auto
    target: strawberry.auto
    @strawberry.field
    async def performance(self, parent: strawberry.Parent[SamplePerformance]) -> list[SamplePerformancePerformanceType]: ...



from io import BytesIO, StringIO
from pathlib import Path

class ExtendedPredictiveModelContent:
    extended_predictive_model_yaml: StringIO
    external_model_json: StringIO | None
    external_model_pmml: StringIO | None
    attached_images: dict[str, BytesIO]
    def __init__(self, extended_predictive_model_yaml: StringIO, external_model_json: StringIO | None = None, external_model_pmml: StringIO | None = None, attached_images: dict[str, BytesIO] = {}) -> None: ...
    def save_to_directory(self, directory: Path, pmml_filename: str | None = None) -> None: ...
    def save_to_tar(self) -> BytesIO: ...
    def save_to_zip(self, pmml_filename: str | None = None) -> BytesIO: ...
    @classmethod
    def load_from_directory(cls, directory: Path) -> ExtendedPredictiveModelContent: ...
    @classmethod
    def load_from_tar(cls, tar_content: BytesIO) -> ExtendedPredictiveModelContent: ...
    @classmethod
    def load_from_zip(cls, zip_content: BytesIO) -> ExtendedPredictiveModelContent: ...



from _typeshed import Incomplete
from tarandm_analytics_utils.predictive_models.attribute_preprocessing.attribute_binning import AttributeBinning as AttributeBinning, AttributeBinningType as AttributeBinningType, AttributeDataType as AttributeDataType, PredictiveModelBinName as PredictiveModelBinName
from tarandm_analytics_utils.predictive_models.attribute_preprocessing.attribute_transformation import AttributeTransformation as AttributeTransformation, AttributeTransformationType as AttributeTransformationType
from tarandm_analytics_utils.utils.base_model import BaseModel as BaseModel
from typing import Any

logger: Incomplete

class PredictiveModelDummyEncodingSingleValue(BaseModel):
    value: str | int | float
    encoded_feature_name: str
    use_for_undefined: bool

class PredictiveModelDummyEncoding(BaseModel):
    attribute: str
    encoding: list[PredictiveModelDummyEncodingSingleValue]
    def check_created_attribute_naming(self) -> PredictiveModelDummyEncoding: ...

class AttributePreprocessing(BaseModel):
    transformations: list[AttributeTransformation]
    binning: list[AttributeBinning]
    dummy_encoding: list[PredictiveModelDummyEncoding]
    def get_transformation_formula(self, attribute: str) -> str: ...
    def add_attribute_transformation(self, transformations: list[dict[str, str]]) -> None: ...
    @classmethod
    def add_attribute_binning(cls, attributes_binning: dict[str, Any]) -> list[AttributeBinning]: ...
    def get_attribute_binning(self, attribute: str) -> AttributeBinning | None: ...
    def apply_attribute_binning(self, attribute: str, attribute_value: str | int | float) -> float | None: ...
    def apply_attribute_binning_batch(self, attribute: str, attribute_values: list) -> list[float] | None: ...
    def attribute_binning_exists(self, attribute: str) -> bool: ...
    def get_all_transformation_attributes(self) -> list[str]: ...
    def get_attributes_created_in_transformations(self) -> list[str]: ...
    def get_all_binning_attributes(self) -> list[str]: ...
    def get_attributes_created_in_binning(self) -> list[str]: ...
    def get_all_dummy_enc_attributes(self) -> list[str]: ...
    def apply_dummy_encoding(self, attribute: str, attribute_value: str | int | float) -> dict[str, int] | None: ...

class AttributePreprocessingType:
    binning: list[AttributeBinningType]
    transformations: list[AttributeTransformationType]



import strawberry
from tarandm_analytics_utils.predictive_models.model_performance.sample_performance import PerformanceMetrics as PerformanceMetrics, SamplePerformance as SamplePerformance, SamplePerformanceType as SamplePerformanceType
from tarandm_analytics_utils.utils.base_model import BaseModel as BaseModel
from typing import Any

class PredictiveModelPerformance(BaseModel):
    sample_performance: list[SamplePerformance]
    performance_metrics: list[PerformanceMetrics]
    @classmethod
    def validate_performance_metric_str(cls, metric: str) -> PerformanceMetrics: ...
    @classmethod
    def validate_performance_metrics(cls, data: Any) -> Any: ...

class PredictiveModelPerformanceType:
    sample_performance: list[SamplePerformanceType]
    performance_metrics: strawberry.auto



import strawberry
from datetime import date
from tarandm_analytics_utils.predictive_models.model_performance.sample_performance import SampleType as SampleType
from tarandm_analytics_utils.utils.base_model import BaseModel as BaseModel
from typing import Any

class ClassFrequency(BaseModel):
    label_class: Any
    number_of_observations: int | None

class ClassFrequencyType:
    number_of_observations: strawberry.auto
    @strawberry.field
    def label_class(self, parent: strawberry.Parent[ClassFrequency]) -> str: ...

class SampleDescription(BaseModel):
    sample_type: SampleType | None
    first_date: date | None
    last_date: date | None
    number_of_observations: int | None
    label_class_frequency: list[ClassFrequency] | None

class SampleDescriptionType:
    sample_type: strawberry.auto
    first_date: strawberry.auto
    last_date: strawberry.auto
    number_of_observations: strawberry.auto
    label_class_frequency: list[ClassFrequencyType] | None



import abc
import strawberry
from _typeshed import Incomplete
from abc import ABC, abstractmethod
from enum import Enum
from io import StringIO
from tarandm_analytics_utils.predictive_models.attribute_preprocessing.attribute_preprocessing import AttributePreprocessing as AttributePreprocessing, AttributePreprocessingType as AttributePreprocessingType
from tarandm_analytics_utils.predictive_models.content import ExtendedPredictiveModelContent as ExtendedPredictiveModelContent
from tarandm_analytics_utils.predictive_models.model_description.model_description import PredictiveModelDescription as PredictiveModelDescription, PredictiveModelDescriptionType as PredictiveModelDescriptionType
from tarandm_analytics_utils.predictive_models.model_monitoring.model_monitoring import PredictiveModelMonitoring as PredictiveModelMonitoring, PredictiveModelMonitoringType as PredictiveModelMonitoringType
from tarandm_analytics_utils.predictive_models.model_performance.model_performance import PredictiveModelPerformance as PredictiveModelPerformance, PredictiveModelPerformanceType as PredictiveModelPerformanceType
from tarandm_analytics_utils.utils.base_model import BaseModel as BaseModel
from tarandm_analytics_utils.utils.dump import safe_dumps_json as safe_dumps_json, safe_loads_yaml as safe_loads_yaml, valid_dumps_yaml as valid_dumps_yaml
from typing import Any

class PredictiveModelType(str, Enum):
    LOGISTIC_REGRESSION = 'LOGISTIC_REGRESSION'
    PMML = 'PMML'
    XGB = 'XGB'
    EXPERT_SCORE = 'EXPERT_SCORE'
    RANDOM_FOREST = 'RANDOM_FOREST'

class AbstractExtendedPredictiveModel(BaseModel, ABC, metaclass=abc.ABCMeta):
    attributes: list[str]
    predictive_model_type: PredictiveModelType | None
    target: str | None
    target_class: str | None
    attribute_preprocessing: AttributePreprocessing
    description: PredictiveModelDescription
    performance: PredictiveModelPerformance
    monitoring: PredictiveModelMonitoring
    def preprocess_attributes(self) -> AbstractExtendedPredictiveModel: ...
    def get_content(self) -> ExtendedPredictiveModelContent: ...
    @classmethod
    def parse_from_content(cls, content: ExtendedPredictiveModelContent) -> AbstractExtendedPredictiveModel: ...
    def get_coefficient(self, attribute: str) -> float | None: ...
    @abstractmethod
    def get_external_model_stringio(self) -> StringIO: ...
    @abstractmethod
    def get_external_model_feature_names(self) -> list[str] | None: ...
    @abstractmethod
    def get_external_model_intercept(self) -> float | None: ...
    @abstractmethod
    def get_external_model_intercept_name(self) -> str | None: ...
    @abstractmethod
    def get_external_model_coefficients(self) -> list[float] | None: ...
    @abstractmethod
    def get_external_model_best_iteration(self) -> int | None: ...
    @abstractmethod
    def get_external_model_number_of_trees(self) -> int | None: ...
    @abstractmethod
    def get_external_model_number_of_attributes(self) -> int | None: ...
    @abstractmethod
    def get_external_model_best_eval_metric(self) -> float | None: ...

class ExtendedPredictiveModel(AbstractExtendedPredictiveModel):
    directory_name: str | None
    external_model: dict[str, Any]
    def get_external_model_stringio(self) -> StringIO: ...
    def get_external_model_feature_names(self) -> list[str] | None: ...
    def get_external_model_intercept(self) -> float | None: ...
    def get_external_model_intercept_name(self) -> str | None: ...
    def get_external_model_coefficients(self) -> list[float] | None: ...
    def get_external_model_best_iteration(self) -> int | None: ...
    def get_external_model_number_of_trees(self) -> int | None: ...
    def get_external_model_number_of_attributes(self) -> int | None: ...
    def get_external_model_best_eval_metric(self) -> float | None: ...

class ExternalPredictiveModelType:
    extended_model: Incomplete
    def __init__(self, extended_model: ExtendedPredictiveModel) -> None: ...
    @strawberry.field
    def coefficients(self) -> list[float] | None: ...
    @strawberry.field
    def intercept(self) -> float | None: ...
    @strawberry.field
    def intercept_name(self) -> str | None: ...
    @strawberry.field
    def best_eval_metric(self) -> float | None: ...
    @strawberry.field
    def best_iteration(self) -> int | None: ...
    @strawberry.field
    def number_of_attributes(self) -> int | None: ...
    @strawberry.field
    def number_of_trees(self) -> int | None: ...

class ExtendedPredictiveModelType:
    directory_name: strawberry.auto
    attributes: strawberry.auto
    predictive_model_type: strawberry.auto
    target: strawberry.auto
    target_class: strawberry.auto
    attribute_preprocessing: AttributePreprocessingType
    description: PredictiveModelDescriptionType
    performance: PredictiveModelPerformanceType
    monitoring: PredictiveModelMonitoringType
    @strawberry.field
    def formula(self, parent: strawberry.Parent[ExtendedPredictiveModel]) -> str | None: ...
    @strawberry.field
    def external_model(self, parent: strawberry.Parent[ExtendedPredictiveModel]) -> ExternalPredictiveModelType: ...



import json
from ruamel.yaml.representer import SafeRepresenter as RuamelSafeRepresenter
from typing import Any, IO

def format_output_data(obj: Any) -> Any: ...
def is_invalid(value: Any) -> bool: ...
def filter_data(data: Any) -> Any: ...

class ValidRepresenter(RuamelSafeRepresenter):
    def represent(self, data: Any) -> None: ...
    def represent_data(self, data: Any) -> Any: ...

def valid_dump_yaml(data: Any, stream: IO[str], **kwds: Any) -> None: ...
def valid_dumps_yaml(data: Any, **kwds: Any) -> str | None: ...
def safe_loads_yaml(content: str) -> Any: ...

class TaranJSONEncoder(json.JSONEncoder):
    def default(self, obj: Any) -> Any: ...

def safe_dumps_json(obj: Any, **kwargs: Any) -> str: ...



from tarandm_analytics_utils.predictive_models.attribute_preprocessing.attribute_binning import AbstractAttributeBin as AbstractAttributeBin, AttributeBinCategorical as AttributeBinCategorical, AttributeBinNumerical as AttributeBinNumerical, AttributeBinning as AttributeBinning, AttributeBinningType as AttributeBinningType, AttributeDataType as AttributeDataType, PredictiveModelBinName as PredictiveModelBinName
from tarandm_analytics_utils.utils.base_model import BaseModel as BaseModel

class PredictiveModelMonitoring(BaseModel):
    binning: list[AttributeBinning]
    predictive_model_output_binning: AttributeBinning | None
    def remove_attribute_binning(self, attribute: str) -> None: ...
    def attribute_binning_exists(self, attribute: str) -> bool: ...
    def get_attribute_binning(self, attribute: str, attribute_dtype: AttributeDataType, attribute_value: str | float) -> AbstractAttributeBin | AttributeBinNumerical | AttributeBinCategorical | None: ...

class PredictiveModelMonitoringType:
    binning: list[AttributeBinningType]
    predictive_model_output_binning: AttributeBinningType | None



# https://python-poetry.org/docs/building-extension-modules/#cython-pyproject

from __future__ import annotations

import os
import shutil
import platform

from pathlib import Path

from Cython.Build import cythonize
from setuptools import Distribution
from setuptools import Extension
from setuptools.command.build_ext import build_ext

# did not work on arm64 architecture clang
# COMPILE_ARGS = ["-march=native", "-O3", "-msse", "-msse2", "-mfma", "-mfpmath=sse"]
COMPILE_ARGS = ["-O3"]
LINK_ARGS = []
INCLUDE_DIRS = []
LIBRARIES = ["m"]
if platform.system().lower() == "windows":
    # O3 doesn't work for Windows
    COMPILE_ARGS = ["/O2", "/arch:AVX2"]
    LIBRARIES = []  # mathematical functions not available on Windows
elif platform.system().lower() == "linux" and platform.machine().lower() == "x86_64":
    # further x86_64 specific optimization flags
    COMPILE_ARGS.extend(["-march=native", "-msse", "-msse2", "-mfma", "-mfpmath=sse"])


def build() -> None:
    # Conditional compilation
    if os.environ.get("USE_CYTHON", "1") == "1":
        extensions = [
            Extension(
                "*",
                ["tarandm_analytics_utils/**/*.py"],
                extra_compile_args=COMPILE_ARGS,
                extra_link_args=LINK_ARGS,
                include_dirs=INCLUDE_DIRS,
                libraries=LIBRARIES,
            )
        ]
        ext_modules = cythonize(
            extensions,
            include_path=INCLUDE_DIRS,
            compiler_directives={"binding": True, "language_level": 3},
        )
    else:
        ext_modules = []  # Skip Cython compilation

    distribution = Distribution({"name": "package", "ext_modules": ext_modules})

    cmd = build_ext(distribution)
    cmd.ensure_finalized()
    cmd.run()

    # Copy built extensions back to the project
    for output in cmd.get_outputs():
        output = Path(output)
        relative_extension = output.relative_to(cmd.build_lib)

        shutil.copyfile(output, relative_extension)
        mode = os.stat(relative_extension).st_mode
        mode |= (mode & 0o444) >> 2
        os.chmod(relative_extension, mode)


if __name__ == "__main__":
    build()



import strawberry
from _typeshed import Incomplete
from abc import ABC
from enum import Enum
from tarandm_analytics_utils.utils.base_model import BaseModel as BaseModel
from typing import Any, Literal

logger: Incomplete

class AttributeDataType(str, Enum):
    CATEGORICAL = 'CATEGORICAL'
    NUMERICAL = 'NUMERICAL'

class PredictiveModelBinName(str, Enum):
    DEFAULT = '0_default'
    DEFINED_DEFAULT = '0_defined_default'
    SYSTEM_DEFAULT = '0_system_default'
    NULL = '1_null'

class AbstractAttributeBin(BaseModel, ABC):
    name: str
    value: float | None
    frequency: float | None
    target_rate: float | None
    id: int
    def __init__(self, **kwargs: Any) -> None: ...

class AttributeBinCategorical(AbstractAttributeBin):
    type: Literal[AttributeDataType.CATEGORICAL]
    categories: list[str] | None
    def __init__(self, **kwargs: Any) -> None: ...

class AttributeBinNumerical(AbstractAttributeBin):
    type: Literal[AttributeDataType.NUMERICAL]
    lower_bound: float | None
    upper_bound: float | None
    def __init__(self, **kwargs: Any) -> None: ...

AttributeBin: Incomplete

class AbstractAttributeBinType:
    name: strawberry.auto
    value: strawberry.auto
    frequency: strawberry.auto
    target_rate: strawberry.auto
    id: strawberry.auto
    @strawberry.field
    async def type(self, parent: strawberry.Parent[AttributeBin]) -> AttributeDataType: ...

class AttributeBinCategoricalType(AbstractAttributeBinType):
    categories: strawberry.auto

class AttributeBinNumericalType(AbstractAttributeBinType):
    lower_bound: strawberry.auto
    upper_bound: strawberry.auto
    @strawberry.field
    async def formatted_lower_bound(self, parent: strawberry.Parent[AttributeBinNumerical]) -> str | None: ...
    @strawberry.field
    async def formatted_upper_bound(self, parent: strawberry.Parent[AttributeBinNumerical]) -> str | None: ...

class AttributeBinning(BaseModel):
    attribute: str
    attribute_data_type: AttributeDataType
    attribute_binning: list[AttributeBin]
    binned_attribute_name: str | None
    def bin_numeric_attribute(self, value: int | float | None) -> float | None: ...
    def bin_categoric_attribute(self, value: str | None) -> float | None: ...

class AttributeBinningType:
    attribute: strawberry.auto
    @strawberry.field
    async def attribute_binning(self, parent: strawberry.Parent[AttributeBinning]) -> list[AttributeBinNumericalType | AttributeBinCategoricalType]: ...



from _typeshed import Incomplete
from pydantic import BaseModel as PydanticBaseModel

class _CyFunctionDetectorMeta(type):
    def __instancecheck__(self, instance: object) -> bool: ...

class CyFunctionDetector(metaclass=_CyFunctionDetectorMeta): ...

class BaseModel(PydanticBaseModel):
    model_config: Incomplete



import strawberry
from datetime import date
from tarandm_analytics_utils.predictive_models.model_description.sample_decription import SampleDescription as SampleDescription, SampleDescriptionType as SampleDescriptionType
from tarandm_analytics_utils.utils.base_model import BaseModel as BaseModel
from typing import Any

class AttachedImage(BaseModel):
    type: str
    filename: str
    image: bytes | None

class AttachedImageType:
    type: strawberry.auto
    filename: strawberry.auto

class PredictiveModelDescription(BaseModel):
    predictive_model_name: str | None
    predictive_model_created: date | None
    sample_metadata: list[SampleDescription] | None
    attribute_description: dict[str, str] | None
    hyperparameters: dict[str, Any] | None
    number_of_trainable_parameters: int | None
    general_notes: dict[str, str] | None
    attached_images: list[AttachedImage] | None
    def add_general_note(self, note: dict | str) -> None: ...
    def remove_general_note(self, note_key: str) -> None: ...

class AttributeDescriptionType:
    attribute_name: str
    description: str

class HyperparameterType:
    name: str
    value: str

class GeneralNoteType:
    title: str
    content: str

class PredictiveModelDescriptionType:
    predictive_model_name: strawberry.auto
    predictive_model_created: strawberry.auto
    number_of_trainable_parameters: strawberry.auto
    attached_images: list[AttachedImageType] | None
    sample_metadata: list[SampleDescriptionType] | None
    @strawberry.field
    async def attribute_description(self, parent: strawberry.Parent[PredictiveModelDescription]) -> list[AttributeDescriptionType]: ...
    @strawberry.field
    async def hyperparameters(self, parent: strawberry.Parent[PredictiveModelDescription]) -> list[HyperparameterType]: ...
    @strawberry.field
    async def general_notes(self, parent: strawberry.Parent[PredictiveModelDescription]) -> list[GeneralNoteType]: ...
    @strawberry.field
    async def eval_metric(self, parent: strawberry.Parent[PredictiveModelDescription]) -> str: ...
